local string = string
local table = table
local io = io

local pkar = require "pocket_archive"


-- Escape magic characters.
local PAS_NS_PTN = pkar.escape_ptn(pkar.PAS_NS)

local M = {models = {}}

-- Parameters that do not get inherited.
local NO_INHERIT = {abstract = true}
local MODEL_PATH = "./config/model/typedef/"


local function camel2snake(src)
    return src
        :gsub("^pas:", "")  -- Strip namespace.
        :gsub("([^^])(%u)", "%1_%2")  -- Uppercase (except initial) to _.
        :lower()
end


M.parse_model = function(mod_id)
    mod_id = mod_id:gsub(PAS_NS_PTN, ""):gsub("par:", "")
    local hierarchy = {}

    local function traverse(mod_id)
        print("traversing:", mod_id)
        local fname = camel2snake (mod_id)
        local model = dofile(MODEL_PATH .. fname .. ".lua")
        --model.id = mod_id
        --print("Model: ")
        --for k, v in pairs(model) do print (k, v) end

        -- Prepend to hierarchy.
        table.insert(hierarchy, 1, model)

        if model.broader then traverse(model.broader) end
    end
    traverse(mod_id)

    local lineage = {}  -- Ordered lineage of types, from ancestor to leaf.
    local types = {}  -- Set of all types.
    for _, mod in ipairs(hierarchy) do
        table.insert(lineage, mod.id)
        types[mod.id] = true
    end

    local function merge(src, dest)
        for k, v in pairs(src) do
            if NO_INHERIT[k] then goto continue end
            if type(v) == "table" then
                dest[k] = dest[k] or {}
                assert(type(dest[k]) == "table")
                merge(v, dest[k])
            else
                dest[k] = v
            end
            ::continue::
        end
    end
    local config = {lineage = lineage, types = types}

    for _, src_config in ipairs(hierarchy) do
        merge(src_config, config)
    end

    return config
end


-- Create a cache for model configurations.
local models_mt = {
    ["__index"] = function (t, model)
        t[model] = M.parse_model(model)
        return t[model]
    end
}
setmetatable(M.models, models_mt)


return M
